from subprocess import call, Popen, PIPE
import json
import os

from tools.functions import utils
from tools.dtos.aws_var import var


def get_profiles():
  f = open(utils.get_config_path("~/.aws/credentials"), 'r')
  lines = f.readlines()
  f.close()
  profiles = []
  for line in lines:
    if "[" in line and "]" in line:
      profiles.append(line.split("[")[1].split("]")[0])
  return profiles

def get_region():
  return [
    "US East (N. Virginia)us-east-1",
    "US East (Ohio)us-east-2",
    "US West (N. California)us-west-1",
    "US West (Oregon)us-west-2",
    "Africa (Cape Town)af-south-1",
    "Asia Pacific (Hong Kong)ap-east-1",
    "Asia Pacific (Jakarta)ap-southeast-3",
    "Asia Pacific (Mumbai)ap-south-1",
    "Asia Pacific (Osaka)ap-northeast-3",
    "Asia Pacific (Seoul)ap-northeast-2",
    "Asia Pacific (Singapore)ap-southeast-1",
    "Asia Pacific (Sydney)ap-southeast-2",
    "Asia Pacific (Tokyo)ap-northeast-1",
    "Canada (Central)ca-central-1",
    "Europe (Frankfurt)eu-central-1",
    "Europe (Ireland)eu-west-1",
    "Europe (London)eu-west-2",
    "Europe (Milan)eu-south-1",
    "Europe (Paris)eu-west-3",
    "Europe (Stockholm)eu-north-1",
    "Middle East (Bahrain)me-south-1",
    "South America (São Paulo)sa-east-1",
  ]

def get_eks():
  process = Popen([
    'aws', 'eks', 
    '--profile', var.profile, 
    '--region', var.region, 
    'list-clusters'
  ], stdout=PIPE, stderr=PIPE)
  out, err = process.communicate()
  response = json.loads(out.decode('ascii','ignore'))
  return response["clusters"]

def set_profile():
  var.profile = utils.render_choices(get_profiles())
  if var.profile != False:
    # os.environ["AWS_PROFILE"] = var.profile
    return True
  return False

def set_region():
  var.region = utils.render_choices(get_region())
  if var.region != False:
    var.region = str(var.region).split(")")[1]
    os.environ["AWS_REGION"] = var.region
    return True
  return False

def set_eks_name():
  var.eks_name = utils.render_choices(get_eks())
  if var.eks_name != False:
    return True
  return False

def set_cluster():
  call([ 
    'aws', 'eks', 
    '--region', var.region, 
    '--profile', var.profile, 
    'update-kubeconfig', 
    '--name', var.eks_name 
  ])


from tools.functions import aws, utils, ssh, jump
import signal

def eksm():
  
  stacks = [
    aws.set_profile,
    aws.set_region,
    aws.set_eks_name
  ]
  if utils.ex_stacks(stacks) :
    aws.set_cluster()  

def sshm():
  stacks = [
    ssh.set_profile,
    ssh.set_ssh
  ]
  if utils.ex_stacks(stacks) :
    ssh.set_profile() 

def jumpm():
  stacks = [
    jump.set_profile,
    jump.set_cidr,
    jump.set_proxy
  ]
  if utils.ex_stacks(stacks) :
    jump.set_cidr()

signal.signal(signal.SIGINT, utils.exist_handler)



from subprocess import call, Popen, PIPE, run

from tools.functions import utils
from tools.dtos.ssh_var import var

def get_profiles():
  f = open(utils.get_config_path("~/.ssh/config"), 'r')
  lines = f.readlines()
  f.close()
  profiles = []
  for line in lines:
    if "Host " in line and "*" not in line:
      profiles.append(line.split("Host ")[1].split("\n")[0])
  return profiles

def set_profile():
  var.profile = utils.render_choices(get_profiles())
  if var.profile != False:
    return True
  return False

def set_ssh():
  run([ 
    'ssh', var.profile 
  ])
  return False



from simple_term_menu import TerminalMenu
import platform
from os.path import expanduser

from tools.dtos.jump_var import var as jump_var
from tools.dtos.utill_var import var as utils_var

def render_choices(options:list):
  options.append("Back")
  terminal_menu = TerminalMenu(options)
  menu_entry_index = terminal_menu.show()
  result = options[menu_entry_index]
  if "Back" in result :
    return False
  return result

def get_executable():
  if platform.system()=='Darwin':
    executable = "/bin/zsh"
  else:
    executable = "/bin/bash"
  return executable

def get_config_path(path):
  path = path.split("~")[1]
  return expanduser("~")+path

def exist_handler(signum, frame):
  if utils_var.allow_exist == False:
    utils_var.allow_exist = True
  else:
    exit(1)

def ex_stacks(stacks: list): 
  i = 0
  while i < len(stacks) and i!= -1:
    r = stacks[i]()
    if r == True :
      i += 1
    elif r == False :
      i-=1
      
  if i != -1:
    return True
  return False


from subprocess import call

from tools.functions import utils
from tools.dtos.jump_var import var
from tools.dtos.utill_var import var as utils_var

def get_profiles():
  f = open(utils.get_config_path("~/.ssh/config"), 'r')
  lines = f.readlines()
  f.close()
  profiles = []
  for line in lines:
    if "Host " in line and "*" not in line:
      profiles.append(line.split("Host ")[1].split("\n")[0])
  return profiles

def get_cidr():
  return [
    "All traffics",
    "Specific cidrs"
  ]

def set_profile():
  var.profile = utils.render_choices(get_profiles())
  if var.profile != False:
    return True
  return False

def set_cidr():
  var.cidr = utils.render_choices(get_cidr())
  if var.cidr != False:
    if "All traffics" in var.cidr:
      var.cidr = "0.0.0.0/0"
    else:
      var.cidr = input("Enter cidr: ")
    return True
  return False
  
def set_proxy():
  utils_var.allow_exist = False
  call([ 
    'sshuttle',
    '-r', 
    var.profile,
    var.cidr, 
    '-vv' 
  ])
  utils_var.allow_exist = True
  return False



from setuptools import setup, find_packages

setup(
    name='devops-auto-tools',
    version='1.1.3',    
    description='A Python project for eks, ssh, jumpm-proxy manager',
    author='Baristi Trieu',
    author_email='ltqtrieu.0204@gmail.com',
    packages=find_packages(),
    license ='MIT',
    install_requires=[
        'simple_term_menu==1.4.1',
        'sshuttle==1.1.1'
    ],
    
    entry_points={
        'console_scripts': [
            'eksm  = tools.main:eksm' ,
            'sshm  = tools.main:sshm' ,
            'jumpm = tools.main:jumpm'
        ]
    },
    
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Science/Research',
        'Operating System :: POSIX :: Linux',        
        'Programming Language :: Python :: 3.8',
    ],
)
